#****************************************************************************
#**
#**  File     :  /cdimage/units/UES0103/UES0103_script.lua
#**  Author(s):  Resin_Smoker
#**
#**  Summary  :  UEF Frigate Script with Advanced Jamming
#**
#**  Copyright  2008 4th Dimension
#****************************************************************************
#**
#**    This script snippet is required to be within the /lua/sim/Units.lua. This is borrowed from the CBFP. It
#**    adds a unit event that's fired when the unit is picked up by a transport or when it's dropped from one.
#**
#**    OnTransportAttach = function(self, attachBone, unit)
#**        oldUnit.OnTransportAttach(self, attachBone, unit)
#**        unit:OnAttachedToTransport(self)
#**    end,
#**
#**    OnAttachedToTransport = function(self, transport)
#**    end,
#**
#**    OnTransportDetach = function(self, attachBone, unit)
#**        oldUnit.OnTransportDetach(self, attachBone, unit)
#**        unit:OnDetachedToTransport(self)
#**    end,
#**
#**    OnDetachedToTransport = function(self, transport)
#**   end,   
#**
#****************************************************************************
local TSeaUnit = import('/lua/terranunits.lua').TSeaUnit
local TAALinkedRailgun = import('/lua/terranweapons.lua').TAALinkedRailgun
local TDFGaussCannonWeapon = import('/lua/terranweapons.lua').TDFGaussCannonWeapon
local Entity = import('/lua/sim/Entity.lua').Entity

UES0103 = Class(TSeaUnit) {

    Weapons = {
        MainGun = Class(TDFGaussCannonWeapon) {
        },
        AAGun = Class(TAALinkedRailgun) {
        },
    },

    OnStopBeingBuilt = function(self,builder,layer)
        TSeaUnit.OnStopBeingBuilt(self,builder,layer)
        ### effects and intel setup
        self.Trash:Add(CreateRotator(self, 'Spinner01', 'y', nil, 360, 0, 180))
        self.Trash:Add(CreateRotator(self, 'Spinner02', 'y', nil, 90, 0, 180))
        self.Trash:Add(CreateRotator(self, 'Spinner03', 'y', nil, -180, 0, -180))
        self.RadarEnt = Entity {}
        self.Trash:Add(self.RadarEnt)
        self.RadarEnt:InitIntel(self:GetArmy(), 'Radar', self:GetBlueprint().Intel.RadarRadius or 75)
        self.RadarEnt:EnableIntel('Radar')
        self.RadarEnt:InitIntel(self:GetArmy(), 'Sonar', self:GetBlueprint().Intel.SonarRadius or 75)
        self.RadarEnt:EnableIntel('Sonar')
        self.RadarEnt:AttachBoneTo(-1, self, 0)

        ### sets up blip tables and starts jamming sequence
        self.BlipTable = {}
        self:SetJammingStatus(self:GetBlueprint().Intel.AltJamming.StartEnabled or true)
    end,

    OnScriptBitSet = function(self, bit)
        TSeaUnit.OnScriptBitSet(self, bit)
        if not self:IsDead() then
            self:SetJammingStatus(false)
        end
    end,

    OnScriptBitClear = function(self, bit)
        TSeaUnit.OnScriptBitClear(self, bit)
        if not self:IsDead() then
            self:SetJammingStatus(true)
        end
    end,

    OnAttachedToTransport = function(self, transport) 
        TSeaUnit.OnAttachedToTransport(self, transport)
        self.PrevJammingStatus = self:GetJammingStatus()
        if self:GetBlueprint().Intel.AltJamming.DisabledInTransport then
            self:SetJammingStatus(false)
        end
    end,

    OnDetachedToTransport = function(self, transport) 
        TSeaUnit.OnDetachedToTransport(self, transport)
        self:SetJammingStatus(self.PrevJammingStatus)
    end,

    SetJammingStatus = function(self, enabled)
        self.JammingActive = enabled
        if enabled then
            self:BlipSpawn()
            self:SetMaintenanceConsumptionActive()
        else
            self:SetMaintenanceConsumptionInactive()
            if self.BlipTable then 
            # Small bit of table manipulation to sort thru all of the avalible blips and remove them
                 if table.getn({self.BlipTable}) > 0 then
                    for k, v in self.BlipTable do 
                        IssueClearCommands({self.BlipTable[k]}) 
                        IssueDestroySelf({self.BlipTable[k]})
                    end 
                end
                self.BlipTable = {}
            end
        end
    end,

    GetJammingStatus = function(self)
        return self.JammingActive
    end,

    BlipSpawn = function(self)
        # put blip BPs here
        local BlipBPs = {
            air = {
                't1_airscout_blip','t1_fighter_blip','t1_bomber_blip','t2_gunship_blip','t2_fighter_bomber_blip',
            },
            ground = {
                't1_anti_air_blip','t1_artillery_blip','t1_bot_blip','t1_scout_blip','t1_tank_blip',
            },
            sea = {
                't1_frigate_blip','t2_sub_blip',
            },
            exp = {
                't4_land_blip',
            },
        }

        local spawnBlip
        local selectBP
        local parentPos = self:GetPosition()
        local bp = self:GetBlueprint().Intel.AltJamming
        local dist = bp.MaxDistance or Random(5, 20)
        local numAirBlips = bp.NumAirBlips or 0
        local numGroundBlips = bp.NumGroundBlips or 0
        local numSeaBlips = bp.NumSeaBlips or 0
        local numExpBlips = bp.NumExpBlips or 0
        local numTotalBlips = numAirBlips + numGroundBlips + numSeaBlips + numExpBlips

        local function CreateBlipUnit(selectBP)
            # creates radar blips
            return CreateUnitHPR(selectBP, self:GetArmy(), parentPos[1]+Random(-dist, dist), parentPos[2], parentPos[3]+ Random(-dist, dist), 0, 0, 0)
        end

        local function defineBlip(spawnBlip)
            # defines the blips made
            spawnBlip:SetCreator(self)
            spawnBlip:SetParent(self)
            spawnBlip:SetAllWeaponsEnabled(false)
            spawnBlip:SetDoNotTarget(true) 
            spawnBlip:SetUnSelectable(true)
            spawnBlip:SetVizToAllies('Never')
            spawnBlip:SetVizToFocusPlayer('Never')
            spawnBlip:SetVizToNeutrals('Never')
            spawnBlip:SetVizToEnemies('Never')
            table.insert(self.BlipTable, spawnBlip) 
            self.Trash:Add(spawnBlip)
            selectBP = nil
        end

        # Selects the blips to make
        if numTotalBlips >= 1 and not self:IsDead() then
            if numAirBlips >= 1 then
                for i = 0, (numAirBlips -1) do
                    selectBP = BlipBPs['air'][ Random( 1, table.getn( BlipBPs['air'] )) ]
                    spawnBlip = CreateBlipUnit(selectBP)
                    spawnBlip = defineBlip(spawnBlip)
                end
            end
            if numGroundBlips >= 1 then
                for i = 0, (numGroundBlips -1) do
                    selectBP = BlipBPs['ground'][ Random( 1, table.getn( BlipBPs['ground'] )) ]
                    spawnBlip = CreateBlipUnit(selectBP)
                    spawnBlip = defineBlip(spawnBlip)
                end
            end
            if numSeaBlips >= 1 then
                for i = 0, (numSeaBlips -1) do
                    selectBP = BlipBPs['sea'][ Random( 1, table.getn( BlipBPs['sea'] )) ]
                    spawnBlip = CreateBlipUnit(selectBP)
                    spawnBlip = defineBlip(spawnBlip)
                end
            end
            if numExpBlips >= 1 then
                for i = 0, (numExpBlips -1) do
                    selectBP = BlipBPs['exp'][ Random( 1, table.getn( BlipBPs['exp'] )) ]
                    spawnBlip = CreateBlipUnit(selectBP)
                    spawnBlip = defineBlip(spawnBlip)
                end
            end
        end
    end,

    OnKilled = function(self, instigator, type, excessDamageRatio)
        self:SetJammingStatus(false)
        self.BlipTable = nil
        TSeaUnit.OnKilled(self, instigator, type, excessDamageRatio)
    end,
}

TypeClass = UES0103