#*************************************************************************
#** 
#**  File     :  t1_frigate_blip_script.lua 
#**  Author   :  Resin_Smoker & Brute51
#**  Summary  :  T1 frigate Blip
#** 
#**  Copyright  2008 4th Dimension
#*************************************************************************
local TSeaUnit = import('/lua/terranunits.lua').TSeaUnit

t1_frigate_blip = Class(TSeaUnit) {
### File pathing and special paramiters called ###########################

    ### Setsup parent call backs between drone and parent
    Parent = nil,

    SetParent = function(self, parent, droneName)
        self.Parent = parent
        self.Drone = droneName
    end,

##########################################################################

    OnCreate = function(self, builder, layer)
        TSeaUnit.OnCreate(self)
        self:ForkThread(self.IssueRandomPatrol)  
        self:ForkThread(self.HeartBeatCheck)  
    end,

    GetParent = function(self)
        return self.Parent or nil
    end,

    IssueRandomPatrol = function(self) 
        if not self:IsDead() and not self.Parent:IsDead() then 
            local parent = self:GetParent() 
            local parentBP = parent:GetBlueprint().Intel.AltJamming 
            local BasePos = parent:CalculateWorldPositionFromRelative({Random(-10, 10), 0, Random(10, 20)})
            local dist = parentBP.MaxDistance - 1
            local numPatrols = Random(2, 5) 
            IssueClearCommands({self}) 
            self:SetSpeedMult(1.0) 
            self:SetAccMult(1.0) 
            self:SetTurnMult(1.0) 
            for k=1,numPatrols do 
                IssuePatrol({self}, Vector( (BasePos[1]+Random(-dist, dist)), BasePos[2], (BasePos[3]+Random(-dist, dist)) ) ) 
            end 
        end 
    end, 

    MoveToparent = function(self) 
        if not self:IsDead() and not self.Parent:IsDead() then 
            local parent = self:GetParent() 
            IssueClearCommands({self}) 
            local destination = parent:CalculateWorldPositionFromRelative({Random(-10, 10), 0, Random(10, 20)})
            IssueMove({self},destination)  
            self:SetSpeedMult(4.0) 
            self:SetAccMult(2.0) 
            self:SetTurnMult(5.0) 
        end 
    end, 

    HeartBeatCheck = function(self) 
        local parent = self:GetParent()
        local maxDist = parent:GetBlueprint().Intel.AltJamming.MaxDistance
        local delay = Random(2, 5)
        local parentPos
        local blipPos
        local oldPos 
        local newPos
        local dest 
        local distTraveled
        while not self:IsDead() and not self.Parent:IsDead() do
            oldPos = self:GetPosition(0)
            WaitSeconds(delay)
            if not self:IsDead() and not self.Parent:IsDead() then 
                newPos = self:GetPosition(0)
                parentPos = parent:GetPosition() 
                blipPos = self:GetPosition()           
                dist = VDist2(blipPos[1], blipPos[3], parentPos[1], parentPos[3]) 
                local maxDist = parent:GetBlueprint().Intel.AltJamming.MaxDistance 
                distTraveled = VDist2(newPos[1], newPos[3], oldPos[1], oldPos[3]) 
                if dist > (maxDist + 5) and self:IsUnitState('Patrolling') then
                    if EntityCategoryContains(categories.AIR, self) then
                        self:IssueRandomPatrol()
                    else
                        self:MoveToparent()
                    end 
                elseif distTraveled <= 1 then
                    dest = parent:CalculateWorldPositionFromRelative({Random(-5, 5), 0, Random(-5, 5)})
                    Warp(self, dest)
                    self:IssueRandomPatrol() 
                end
            end 
        end      
    end,

    OnMotionHorzEventChange = function(self, new, old) 
        if not self:IsDead() and not self.Parent:IsDead() then 
            if new == 'Stopped' or new == 'Stopping' then
                local parent = self:GetParent() 
                local parentPos = parent:GetPosition()  
                pos = self:GetPosition() 
                dist = VDist2(pos[1], pos[3], parentPos[1], parentPos[3])  
                local maxDist = parent:GetBlueprint().Intel.AltJamming.MaxDistance 
                if dist > (maxDist + 1) then            
                    if EntityCategoryContains(categories.AIR, self) then
                        IssueClearCommands({self})   
                        IssueGuard({self}, parent) 
                    else
                        self:MoveToparent()
                    end 
                elseif dist <= maxDist and not self:IsUnitState('Patrolling') then
                    self:IssueRandomPatrol() 
                end 
            end
        end      
        TSeaUnit.OnMotionHorzEventChange(self, new, old) 
    end,
}
TypeClass = t1_frigate_blip


